@@ -1,5 +1,5 @@
 package PostScript::File;
-our $VERSION = '1.03';
+our $VERSION = '1.04';
 use 5.008;
 use strict;
 use warnings;
@@ -18,7 +18,7 @@ our @EXPORT_OK = qw(check_tilde check_file incpage_label incpage_roman
 
 # global constants
 my $rmspace   = qr(^\s+)m;          # remove leading spaces
-my $rmcomment = qr(^\s+\(% .*\)?)m; # remove single line comments
+my $rmcomment = qr(^\s*\(%(?![!%]).*\n\)?)m; # remove single line comments
 
 =head1 NAME
 
@@ -26,8 +26,8 @@ PostScript::File - Base class for creating Adobe PostScript files
 
 =head1 VERSION
 
-This document describes version 1.03 of PostScript::File,
-released October 20, 2009.
+This document describes version 1.04 of PostScript::File,
+released October 22, 2009.
 
 
 =head1 SYNOPSIS
@@ -484,6 +484,9 @@ A6, A7, A8, A9, B0, B1, B2, B3, B4, B5, B6, B7, B8, B9, B10, Executive, Folio, '
 Legal, 'US-Legal', Tabloid, 'SuperB', Ledger, 'Comm #10 Envelope', 'Envelope-Monarch', 'Envelope-DL',
 'Envelope-C5', 'EuroPostcard'.  (Default: "A4")
 
+You can also give a string in the form 'WIDTHxHEIGHT', where WIDTH and
+HEIGHT are numbers (in points).  This sets the paper size to "Custom".
+
 This also sets C<width> and C<height>.  B<get_paper> returns the value set here.
 
 =head3 right
@@ -629,8 +632,8 @@ Set the label (text or number) for the initial page.  See L</set_page_label>.  (
 
 =head3 strip
 
-Set whether the postscript code is filtered.  C<space> strips leading spaces so the user can indent freely
-without increasing the file size.  C<comments> remove lines beginning with '%' as well.  (Default: "space")
+Set whether the PostScript code is filtered.  C<space> strips leading spaces so the user can indent freely
+without increasing the file size.  C<comments> remove lines beginning with '%' as well.  C<none> does no filtering.  (Default: "space")
 
 =cut
 
@@ -671,7 +674,7 @@ sub pre_pages {
         $o->{DocSupplied} .= "\%\%+ Encoded_Fonts\n";
         my $encoding = $o->{reencode};
         my $ext = $o->{font_suffix};
-        ($fonts .= <<END_FONTS) =~ s/$o->{strip}//gm;
+        $fonts .= $o->_here_doc(<<END_FONTS);
         \%\%BeginResource: Encoded_Fonts
             /STARTDIFFENC { mark } bind def
             /ENDDIFFENC {
@@ -774,23 +777,23 @@ END_FONTS
     my $hostname = hostname();
     my $postscript = $o->{eps} ? "\%!PS-Adobe-3.0 EPSF-3.0\n" : "\%!PS-Adobe-3.0\n";
     if ($o->{eps}) {
-        ($postscript .= <<END_EPS) =~ s/$o->{strip}//gm;
+        $postscript .= $o->_here_doc(<<END_EPS);
         \%\%BoundingBox: $o->{bbox}[0] $o->{bbox}[1] $o->{bbox}[2] $o->{bbox}[3]
 END_EPS
     }
     if ($o->{headings}) {
-        ($postscript .= <<END_TITLES) =~ s/$o->{strip}//gm;
+        $postscript .= $o->_here_doc(<<END_TITLES);
         \%\%For: $user\@$hostname
         \%\%Creator: Perl module ${\( ref $o )} v$PostScript::File::VERSION
         \%\%CreationDate: ${\( scalar localtime )}
 END_TITLES
-        ($postscript .= <<END_PS_ONLY) =~ s/$o->{strip}//gm if (not $o->{eps});
+        $postscript .= $o->_here_doc(<<END_PS_ONLY) if (not $o->{eps});
         \%\%DocumentMedia: $o->{paper} $o->{width} $o->{height} 80 ( ) ( )
 END_PS_ONLY
     }
 
     my $landscapefn = "";
-    ($landscapefn .= <<END_LANDSCAPE) =~ s/$o->{strip}//gm if ($landscape);
+    $landscapefn .= $o->_here_doc(<<END_LANDSCAPE) if ($landscape);
                 % Rotate page 90 degrees
                 % _ => _
                 /landscape {
@@ -800,7 +803,7 @@ END_PS_ONLY
 END_LANDSCAPE
 
     my $clipfn = "";
-    ($clipfn .= <<END_CLIPPING) =~ s/$o->{strip}//gm if ($clipping);
+    $clipfn .= $o->_here_doc(<<END_CLIPPING) if ($clipping);
                 % Draw box as clipping path
                 % x0 y0 x1 y1 => _
                 /cliptobox {
@@ -818,7 +821,7 @@ END_LANDSCAPE
 END_CLIPPING
 
     my $errorfn = "";
-    ($errorfn .= <<END_ERRORS) =~ s/$o->{strip}//gm if ($o->{errors});
+    $errorfn .= $o->_here_doc(<<END_ERRORS) if ($o->{errors});
         /errx $o->{errx} def
         /erry $o->{erry} def
         /errmsg ($o->{errmsg}) def
@@ -852,7 +855,7 @@ END_CLIPPING
 END_ERRORS
 
     my $debugfn = "";
-    ($debugfn .= <<END_DEBUG_ON) =~ s/$o->{strip}//gm if ($o->{debug});
+    $debugfn .= $o->_here_doc(<<END_DEBUG_ON) if ($o->{debug});
         /debugdict 25 dict def
         debugdict begin
 
@@ -1015,7 +1018,7 @@ END_ERRORS
         end
 END_DEBUG_ON
 
-    ($debugfn .= <<END_DEBUG_OFF) =~ s/$o->{strip}//gm if (defined($o->{debug}) and not $o->{debug});
+    $debugfn .= $o->_here_doc(<<END_DEBUG_OFF) if (defined($o->{debug}) and not $o->{debug});
         % Define out the db_ functions
         /debugdict 25 dict def
         debugdict begin
@@ -1035,12 +1038,12 @@ END_DEBUG_OFF
     my $supplied = "";
     if ($landscapefn or $clipfn or $errorfn or $debugfn) {
         $o->{DocSupplied} .= "\%\%+ procset PostScript_File\n";
-        ($supplied .= <<END_DOC_SUPPLIED) =~ s/$o->{strip}//gm;
+        $supplied .= $o->_here_doc(<<END_DOC_SUPPLIED);
             \%\%BeginProcSet: PostScript_File
-                $landscapefn
-                $clipfn
-                $errorfn
-                $debugfn
+            $landscapefn
+            $clipfn
+            $errorfn
+            $debugfn
             \%\%EndProcSet
 END_DOC_SUPPLIED
     }
@@ -1059,22 +1062,22 @@ END_DOC_SUPPLIED
 
     $postscript .= $o->{Preview} if ($o->{Preview});
 
-    ($postscript .= <<END_DEFAULTS) =~ s/$o->{strip}//gm if ($o->{Defaults});
+    $postscript .= $o->_here_doc(<<END_DEFAULTS) if ($o->{Defaults});
         \%\%BeginDefaults
             $o->{Defaults}
         \%\%EndDefaults
 END_DEFAULTS
 
-    ($postscript .= <<END_PROLOG) =~ s/$o->{strip}//gm;
+    $postscript .= $o->_here_doc(<<END_PROLOG);
         \%\%BeginProlog
-            $supplied
-            $fonts
-            $o->{Resources}
-            $o->{Functions}
+        $supplied
+        $fonts
+        $o->{Resources}
+        $o->{Functions}
         \%\%EndProlog
 END_PROLOG
 
-    ($postscript .= <<END_SETUP) =~ s/$o->{strip}//gm if ($o->{Setup});
+    $postscript .= $o->_here_doc(<<END_SETUP) if ($o->{Setup});
         \%\%BeginSetup
             $o->{Setup}
         \%\%EndSetup
@@ -1087,7 +1090,7 @@ sub post_pages {
     my $o = shift;
     my $postscript = "";
 
-    ($postscript .= <<END_TRAILER) =~ s/$o->{strip}//gm if ($o->{Trailer});
+    $postscript .= $o->_here_doc(<<END_TRAILER) if ($o->{Trailer});
         \%\%Trailer
         $o->{Trailer}
 END_TRAILER
@@ -1176,7 +1179,7 @@ END_DEBUG_END
                 $pagebb = "\%\%PageBoundingBox: $pbox[0] $pbox[1] $pbox[2] $pbox[3]";
             }
             my $cliptobox = $o->{pageclip}[$p] ? "$pbox[0] $pbox[1] $pbox[2] $pbox[3] cliptobox" : "";
-            ($postscript .= <<END_PAGE_SETUP) =~ s/$o->{strip}//gm;
+            $postscript .= $o->_here_doc(<<END_PAGE_SETUP);
                 \%\%Page: $o->{page}->[$p] ${\($p+1)}
                 $pagebb
                 \%\%BeginPageSetup
@@ -1188,7 +1191,7 @@ END_DEBUG_END
                 \%\%EndPageSetup
 END_PAGE_SETUP
             $postscript .= $o->{Pages}->[$p];
-            ($postscript .= <<END_PAGE_TRAILER) =~ s/$o->{strip}//gm;
+            $postscript .= $o->_here_doc(<<END_PAGE_TRAILER);
                 \%\%PageTrailer
                     $o->{PageTrailer}
                     $debugend
@@ -1345,7 +1348,14 @@ sub get_paper {
 sub set_paper {
     my $o = shift;
     my $paper = shift || "A4";
-    my ($width, $height) = split(/\s+/, $size{lc($paper)});
+    my ($width, $height) = split(/\s+/, $size{lc($paper)} || '');
+
+    if (not $height and $paper =~ /^(\d+(?:\.\d+)?)\s*x\s*(\d+(?:\.\d+)?)$/i) {
+      $width  = $1;
+      $height = $2;
+      $paper  = 'Custom';
+    } # end if $paper is 'WIDTH x HEIGHT'
+
     if ($height) {
         $o->{paper} = $paper;
         $o->{width} = $width;
@@ -1755,7 +1765,7 @@ sub get_comments {
 
 sub add_comment {
     my ($o, $entry) = @_;
-    $o->{Comments} = "\%\%$entry\n" if defined($entry);
+    $o->{Comments} .= "\%\%$entry\n" if defined($entry);
 }
 
 =head2 get_comments()
@@ -1785,7 +1795,7 @@ sub add_preview {
     my ($o, $width, $height, $depth, $lines, $entry) = @_;
     if (defined $entry) {
         $entry =~ s/$o->{strip}//gm;
-        ($o->{Preview} = <<END_PREVIEW) =~ s/$o->{strip}//gm;
+        $o->{Preview} = $o->_here_doc(<<END_PREVIEW);
             \%\%BeginPreview: $width $height $depth $lines
                 $entry
             \%\%EndPreview
@@ -1840,7 +1850,7 @@ sub add_resource {
         $resource =~ s/$o->{strip}//gm;
         $o->{DocSupplied} .= "\%\%+ $supplied_type{$type} $name\n"
             if defined $supplied_type{$type};
-        ($o->{Resources} = <<END_USER_RESOURCE) =~ s/$o->{strip}//gm;
+        $o->{Resources} = $o->_here_doc(<<END_USER_RESOURCE);
             \%\%Begin${type}: $name $params
             $resource
             \%\%End$type
@@ -1896,7 +1906,7 @@ sub add_function {
     if (defined($name) and defined($entry)) {
         $entry =~ s/$o->{strip}//gm;
         $o->{DocSupplied} .= "\%\%+ procset $name\n";
-        ($o->{Functions} .= <<END_USER_FUNCTIONS) =~ s/$o->{strip}//gm;
+        $o->{Functions} .= $o->_here_doc(<<END_USER_FUNCTIONS);
             \%\%BeginProcSet: $name
             $entry
             \%\%EndProcSet
@@ -1960,7 +1970,7 @@ sub embed_document
 {
   my ($o, $filename) = @_;
 
-  my $id = $o->pstr($filename);
+  my $id = $o->pstr(substr($filename, -234), 1); # in case it's long
   $o->{DocSupplied} .= "%%+ file $id\n"
       unless index($o->{DocSupplied}, "%%+ file $id\n") >= 0;
 
@@ -2269,6 +2279,24 @@ sub clip_bounding_box {
     $o->{clipcmd} = "clip";
 }
 
+# Strip leading spaces off a here document:
+
+sub _here_doc
+{
+  my ($o, $text) = @_;
+
+  if ($o->{strip}) {
+    $text =~ s/$o->{strip}//gm;
+  } elsif ($text =~ /^([ \t]+)/) {
+    my $space = $1;
+
+    $text =~ s/^$space//gm;
+    $text =~ s/^[ \t]+\n/\n/gm;
+  } # end elsif no strip but $text is indented
+
+  $text;
+} # end _here_doc
+
 =head1 EXPORTED FUNCTIONS
 
 No functions are exported by default, they must be named as required.
@@ -2441,13 +2469,23 @@ my $specialKeys = join '', keys %special;
 sub pstr {
   shift if @_ == 2;             # We were called as a method
   my $string = shift;
+  my $nowrap = shift;
   $string =~ s/([$specialKeys])/$special{$1}/go;
-  "($string)";
+  $string = "($string)";
+  # A PostScript file should not have more than 255 chars per line:
+  $string =~ s/(.{240}[^\\])/$1\\\n/g unless $nowrap;
+  $string =~ s/^([ %])/\\$1/mg; # Make sure it doesn't get stripped
+
+  $string;
 } # end pstr
 
-=head2 pstr( string )
+=head2 pstr( string, [nowrap] )
 
-Converts the string to a string representation suitable for postscript code.
+Converts the string to a string representation suitable for postscript
+code.  If the result is more than 240 characters, it will be broken
+into multiple lines unless the optional nowrap parameter is true.  (A
+PostScript file should not contain lines with more than 255
+characters.)
 
 This may also be called as a class or object method.
 
